<?php
declare(strict_types=1);

namespace Inti\Wp\Provider\Container;

use Doctrine\Common\Collections\ArrayCollection;
use Inti\Wp\Provider\Service\ProviderCollection;
use Inti\Wp\Provider\Service\ProviderManager;
use Psr\Container\ContainerInterface;
use function array_map;
use function is_string;

/**
 * Class ProviderCollectionFactory
 *
 * @package Inti\Wp\Provider\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
final class ProviderCollectionFactory
{
    public function __invoke(ContainerInterface $container): ProviderCollection
    {
        $config = $container->has('config') ? $container->get('config') : [];
        /** @var ProviderManager $manager */
        $manager = $container->get(ProviderManager::class);

        $collection = new ArrayCollection(array_map(function ($provider) use ($manager) {
            return is_string($provider) ? $manager->get($provider) : $provider;
        }, $config['wp']['providers'] ?? []));

        return new ProviderCollection($collection);
    }
}
