<?php
declare(strict_types=1);

namespace Inti\Wp\Provider\Service;

use Doctrine\Common\Collections\Collection;
use Inti\Wp\Provider\ObjectProviderInterface;
use Inti\Wp\Provider\ProviderInterface;

/**
 * Class ProviderCollection
 *
 * @package Inti\Wp\Provider\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
final class ProviderCollection
{
    /** @var Collection|ProviderInterface[] */
    private Collection $providers;

    /**
     * Konstruktor.
     *
     * @param Collection $providers
     */
    public function __construct(Collection $providers)
    {
        $this->providers = $providers;
    }

    public function getProvider(string $interface, string $key): ?ProviderInterface
    {
        $providers = $this->getProviders($interface, $key);
        return count($providers) > 0 ? $providers[0] : null;
    }

    /**
     * @param string|null $interface
     * @param string|null $key
     * @return Collection|ProviderInterface[]|ObjectProviderInterface[]
     */
    public function getProviders(?string $interface = null, ?string $key = null): Collection
    {
        $providers = $this->providers;

        if (isset($interface)) {
            $providers = $providers->filter(function (ProviderInterface $provider) use ($interface) {
                return $provider instanceof $interface;
            });
        }

        if (isset($key)) {
            $providers = $providers->filter(function (ProviderInterface $provider) use ($key) {
                return $provider->getObjectType()->getKey() === $key;
            });
        }

        return $providers;
    }
}
