<?php
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Mock;

use Inti\Wp\Provider\Post\ProviderInterface;
use Inti\Wp\Provider\Type\ObjectTypeInterface;
use function array_combine;
use function array_filter;
use function array_map;
use function in_array;
use function preg_match;
use function preg_quote;

/**
 * Class PostProvider
 *
 * @package IntiTest\Wp\Provider\Mock
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
final class PostProvider extends PostMetadataProvider implements ProviderInterface
{
    private array $posts = [
        [1, 'First post', 'first-post'],
        [2, 'Second post', 'second-post'],
        [3, 'Third post', 'third-post'],
    ];

    private array $fields = [
        'ID',
        'post_title',
        'post_name',
    ];

    public function getObjectType(): ObjectTypeInterface
    {
        return new CustomPostType();
    }

    public function count(array $criteria = []): int
    {
        return count($this->posts);
    }

    public function findByIds(array $ids): array
    {
        $posts = array_filter($this->posts, function (array $row) use ($ids) {
            return in_array($row[0], $ids);
        });

        return array_map([$this, 'loadRow'], $posts);
    }

    public function findBySlug(string $slug): ?array
    {
        $posts = array_filter($this->posts, function (array $row) use ($slug) {
            return $slug === $row[2];
        });

        return count($posts) > 0
            ? $this->loadRow($posts[0])
            : null;
    }

    public function getAll(int $limit, int $offset = 0): array
    {
        return array_map([$this, 'loadRow'], $this->posts);
    }

    public function search(string $query, int $limit, int $offset = 0): array
    {
        return array_map([$this, 'loadRow'], array_filter($this->posts, function (array $row) use ($query) {
            return preg_match('/' . preg_quote($query, '/') . '/', $row[1]);
        }));
    }

    public function loadRow(array $row): array
    {
        return array_combine($this->fields, $row);
    }
}
