<?php
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Plugin;

use Inti\Wp\Provider\Plugin\ProvideMetadata;
use Inti\Wp\Provider\Plugin\ProvidePosts;
use IntiTest\Wp\Provider\Mock\PostProvider;
use IntiTest\Wp\Provider\TestCaseTrait;
use Laminas\Stdlib\ArrayUtils;
use PHPUnit\Framework\TestCase;
use WP_Post_Type;

use function get_post;
use function get_post_meta;
use function get_posts;
use function wp_count_posts;

/**
 * Class ProvidePostsTest
 *
 * @package IntiTest\Wp\Provider\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-bartog
 *
 * @runTestsInSeparateProcesses
 */
final class ProvidePostsTest extends TestCase
{
    use TestCaseTrait;

    protected function setUp(): void
    {
        $this->setUpContainer();
        $this->setUpWordPress();

        $this->config = ArrayUtils::merge($this->config, [
            'wp' => [
                'plugins' => [
                    ProvidePosts::class,
                    ProvideMetadata::class,
                ],
                'providers' => [
                    PostProvider::class,
                ],
            ],
        ]);
    }

    public function testWillRegisterCustomPostType()
    {
        $this->runApplication();

        $type = get_post_type_object('custom_post');
        $this->assertInstanceOf(WP_Post_Type::class, $type);

        $this->assertEquals('Custom Posts', $type->label);
        $this->assertEquals('some description', $type->description);
        $this->assertTrue($type->public);
    }

    public function testCanProvidePosts()
    {
        $this->runApplication();

        $posts = get_posts([
            'post_type' => 'custom_post'
        ]);

        $this->assertCount(3, $posts);
    }

    public function testCanGetPostById()
    {
        $this->runApplication();

        $post = get_post(10000001);
        $this->assertEquals(10000001, $post->ID);
        $this->assertEquals('custom_post', $post->post_type);
        $this->assertEquals('First post', $post->post_title);
    }

    public function testCanCountPosts()
    {
        $this->runApplication();

        $count = wp_count_posts('custom_post');
        $this->assertEquals(3, $count->publish);
    }

    public function testCanProvideMetadataForProvidedPosts()
    {
        $this->runApplication();

        $post = get_post(10000001);
        $metadata = get_post_meta($post->ID, '_meta_key1', true);

        $this->assertEquals('meta value 1-1', $metadata);
    }
}
