<?php
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Plugin;

use Inti\Wp\Provider\Plugin\RegisterTypes;
use IntiTest\Wp\Provider\Mock\TermProvider;
use IntiTest\Wp\Provider\TestCaseTrait;
use Laminas\Stdlib\ArrayUtils;
use PHPUnit\Framework\TestCase;
use WP_Taxonomy;
use function get_taxonomy;

/**
 * Class RegisterTypesTest
 *
 * @package IntiTest\Wp\Provider\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-bartog
 *
 * @runTestsInSeparateProcesses
 */
final class RegisterTypesTest extends TestCase
{
    use TestCaseTrait;

    protected function setUp(): void
    {
        $this->setUpContainer();
        $this->setUpWordPress();

        $this->config = ArrayUtils::merge($this->config, [
            'wp' => [
                'plugins' => [RegisterTypes::class],
            ],
        ]);
    }

    public function testWillRegisterCustomTaxonomy()
    {
        $this->runApplicationWithProvider(new TermProvider());

        $taxonomy = get_taxonomy('custom_taxonomy');
        $this->assertInstanceOf(WP_Taxonomy::class, $taxonomy);

        $this->assertEquals('Custom Terms', $taxonomy->label);
        $this->assertEquals('some description', $taxonomy->description);
        $this->assertFalse($taxonomy->public);
        $this->assertEquals(['custom_post'], $taxonomy->object_type);
    }
}
