<?php
declare(strict_types=1);

namespace IntiTest\Wp\Provider;

use Inti\Test\TestCase\ApplicationTestCase;
use Inti\Wp\Provider\ConfigProvider;
use Inti\Wp\Provider\ProviderInterface;
use Laminas\Stdlib\ArrayUtils;
use Psr\Http\Message\ResponseInterface;

use function define;
use function defined;
use function realpath;

/**
 * Trait TestCaseTrait
 *
 * @package IntiTest\Wp\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
trait TestCaseTrait
{
    use ApplicationTestCase;

    protected array $providers = [
        \Inti\Wp\ConfigProvider::class,
        ConfigProvider::class,
    ];

    protected array $config = [
        'db_connections' => [
            'default' => [
                'host' => 'localhost',
                'user' => 'root',
                'password' => 'password',
                'dbname' => 'inti_wp',
            ],
        ],
    ];

    protected function setUpWordPress()
    {
        if (!defined('ABSPATH')) {
            define('ABSPATH', realpath(__DIR__ . '/../wordpress') . '/');
        }

        if (!defined('WP_DEBUG')) {
            define('WP_DEBUG', true);
        }
    }

    public function runApplicationWithPlugin(string $plugin, ProviderInterface $provider = null, array $config = []): ResponseInterface
    {
        $config['wp']['plugins'] = [$plugin];

        return $provider instanceof ProviderInterface
            ? $this->runApplicationWithProvider($provider, $config)
            : $this->runApplication($config);
    }

    public function runApplicationWithProvider(ProviderInterface $provider, array $config = []): ResponseInterface
    {
        return $this->runApplication(ArrayUtils::merge($config, [
            'wp' => [
                'providers' => [$provider],
            ],
        ]));
    }
}
